module Turboslugs
  def self.included(base)
    base.send(:include, InstanceMethods)
    base.extend(ClassMethods)
    base.class_eval do
      validates :slug, presence: true
      before_validation :generate_slug

      self.restricted_slugs = [:new]
    end
  end

  module InstanceMethods
    def to_param
      self.slug
    end

    private

    def generate_slug
      return unless !!slug_base

      base = slug_base
      slug = slugify(base)
      counter = 1

      while restricted?(slug) || taken?(slug)
        slug = slugify(base)
        counter += 1

        slug += "_#{counter}"
      end

      self.slug = slug

      slug
    end

    def restricted? slug
      self.class.restricted_slugs.include?(slug.to_sym)
    end

    def taken? slug
      match = self.class.find_by(slug: slug)

      !!match && (match != self)
    end

    def slug_base
      base = self.class.slug_base
      send base if base
    end

    def slugify string
      string.downcase.gsub(/\s/, '_').gsub(/\W/, '')
    end
  end

  module ClassMethods
    def slug_from base_attribute
      @@slug_base = base_attribute
    end

    def slug_base
      @@slug_base
    end

    def restricted_slugs= other
      @@restricted_slugs = other
    end

    def restricted_slugs
      @@restricted_slugs
    end
  end
end
